/* ------------------------------------------------------------------------- */
/*
 *  socket.h
 *
 *  Copyright (c) 2004 - 2006 "clown". All rights reserved.
 *
 *  Permission to copy, use, modify and distribute this software is
 *  granted provided this copyright notice appears in all copies.
 *  This software is provided "as is" without express or implied
 *  warranty, and with no claim as to its suitability for any purpose.
 *
 *  Last-modified: Sun 19 Feb 2006 13:35:00 JST
 */
/* ------------------------------------------------------------------------- */
#ifndef CLX_SOCKET_H
#define CLX_SOCKET_H

#include <cstdlib>
#include <cstring>
#include <string>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include "clx/inet_address.h"

namespace clx {
	typedef int socket_type;
	typedef socklen_t socklen_type;
	
	/* --------------------------------------------------------------------- */
	//  basic_socket
	/* --------------------------------------------------------------------- */
	template <
		int Type,
		int Family = AF_INET
	>
	class basic_socket {
	public:
		// constructor and destructor
		basic_socket() : s_(-1) {}
		explicit basic_socket(socket_type s) : s_(s) {}
		
		virtual ~basic_socket() { this->close(); }
		
		basic_socket& operator=(socket_type s) {
			this->socket(s);
			return *this;
		}
		
		// operator
		virtual socket_type open() {
			s_ = ::socket(Family, Type, 0);
			return s_;
		}
		
		virtual void close() {
			if (s_ >= 0) {
				::close(s_);
				s_ = -1;
			}
		}
		
		socket_type socket() const { return s_; }
		
		socket_type socket(socket_type s) {
			// another socket has been already open.
			if (s_ >= 0) return -1;
			s_ = s;
			return s_;
		}
		
	private:
		socket_type s_;
	};
	
	/* --------------------------------------------------------------------- */
	//  basic_socket_address
	/* --------------------------------------------------------------------- */
	template <int Family = AF_INET> class basic_socket_address {
	public:
		typedef char char_type;
		typedef unsigned int size_type;
		typedef std::string string_type;
		typedef struct sockaddr_in inet_type;
		typedef struct sockaddr base_type;
		
		// constructor and destructor
		basic_socket_address() : addr_() {}
		explicit basic_socket_address(const inet_type& addr) : addr_(addr) {}
		
		explicit basic_socket_address(int port) : addr_() {
			this->assign(port);
		}
		
		explicit basic_socket_address(const string_type& host, int port) : addr_() {
			this->assign(host, port);
		}
		
		virtual ~basic_socket_address() {}
		
		basic_socket_address& operator=(const inet_type& addr) {
			addr_ = addr;
			return *this;
		}
		
		// operator
		virtual bool assign(int port) {
			memset((char*)&addr_, 0, sizeof(addr_));
			addr_.sin_family = Family;
			addr_.sin_port = htons(port);
			addr_.sin_addr.s_addr = htonl(INADDR_ANY);
			return true;
		}
		
		virtual bool assign(const char_type* host, int port) {
			inet_address::address_type tmp;
			if (!::inet_aton(host, &tmp)) {
				inet_address inet;
				inet.lookup(host);
				if (inet.empty()) return false;
				tmp = inet.at(0);
			}
			
			memset(&addr_, 0, sizeof(addr_));
			addr_.sin_family = Family;
			addr_.sin_port = htons(port);
			bcopy(&tmp, (char*)&addr_.sin_addr, sizeof(tmp));
			return true;
		}
		
		virtual bool assign(const string_type& host, int port) {
			return this->assign(host.c_str(), port);
		}
		
		// get information about socket address
		const inet_type& reference() const { return addr_; }
		const inet_type* data() const { return &addr_; }
		const base_type* base_data() const { return (base_type*)this->data(); }
		size_type size() const { return sizeof(addr_); }
		
	private:
		inet_type addr_;
	};
	
	typedef basic_socket_address<> socket_address;
}

#endif // CLX_SOCKET_H
